/* $Id: planets.h,v 1.12 2000/02/01 21:22:02 rsmith Exp $
 * This file is part of xplanets
 * Copyright (C) 1998-2000  R.F. Smith
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * You can contact the author at the following addresses:
 *      email: rsmith@xs4all.nl
 * snail-mail: R.F. Smith
 *             Dr. Hermansweg 36
 *             5624 HR Eindhoven
 *             The Netherlands
 *
 * $Log: planets.h,v $
 * Revision 1.12  2000/02/01 21:22:02  rsmith
 * w2s now exported as pl_world_to_screen
 *
 * Revision 1.11  2000/01/30 16:19:36  rsmith
 * Added screen size of planet to planet_t struct.
 *
 * Revision 1.10  2000/01/29 19:10:48  rsmith
 * Moved rel_pos and rel_speed arrays to planets.*.
 * Added pl_update_rel.
 *
 * Revision 1.9  2000/01/28 19:42:08  rsmith
 * Changed ship.dv to ship.acc.
 * Release 0.4.0
 *
 * Revision 1.8  2000/01/25 21:39:11  rsmith
 * Removed gv and av from ship_t. Added dir and dv and engine_on to ship_t.
 * Removed the superfluous pl_set_ship_acc function. These changes implement
 * the more realistic "delta v" vector steering mechanism.
 *
 * Revision 1.7  2000/01/23 21:33:05  rsmith
 * Added fields for synchronous orbit data to planet_t strcuture.
 *
 * Revision 1.6  2000/01/23 11:11:45  rsmith
 * Changed the view setting interface.
 *
 * Revision 1.5  2000/01/14 20:02:32  rsmith
 * - Removed color from planet_t.
 * - Added rel_t.
 * - Changed pl_distance
 * - Added pl_rel
 *
 * Revision 1.4  2000/01/09 10:36:22  rsmith
 * Frozen for release 0.3.0
 *
 * Revision 1.3  2000/01/07 13:46:15  rsmith
 * This version corresponds with Revision 1.4 of planets.c.
 *
 */

#ifndef _PLANETS_H
#define _PLANETS_H

#define NUM_PLANETS 10  /* including the sun, #0! */

/* Coordinate pair (integer) */
typedef struct {
  int x,y;
} ci_t;

/* Coordinate pair (double) */
typedef struct {
  double x,y;
} cd_t;

/* Polar coordinates (double) */
typedef struct {
  double len, phi;
} cdp_t;


/* Relative position or speed */
typedef struct {
  double x, y;          /* cartesian coordinates */
  double len, phi;      /* polar coordinates */
} rel_t;         

/* Data to describe a spaceship */
typedef struct {
  cd_t pos;             /* world coordinates [meters] */
  cd_t speed;           /* velocity [m/s] */
  ci_t screen;          /* screen position */
  int on_screen;        /* visible? */
  double dir;           /* directions of accelleration [rad] */
  double acc;           /* accelleration [m/s^2] */
  int engine_on;        /* <>0 if engine is on. */
} ship_t;

/* Data to describe a planet */
typedef struct {
  cd_t pos;             /* world coordinates [meters] */
  cd_t speed;           /* velocity [m/s] */
  ci_t screen;          /* screen position */
  int r_screen;         /* planetary radius on screen */
  int on_screen;        /* visible? */
  double mass;          /* planetary mass [kg] */
  double radius;        /* planetary radius [m] */
  double orbit, angle;  /* orbital radius [m] and angle [rad]; */
  double w;             /* angular velocity [radians/s] */
  double w_geo;         /* sync. orbit angular velocity [radians/s] */
  double r_geo;         /* sync. orbit radius [m] */
  char *name;           /* the planet's name */
} planet_t;

/* minumum on-screen planetary radius */
extern int pl_min_screen_size;

/* The following variables should *NOT* be changed outside the functions!
 * They ara provided for information purposes. Mess them up and you'll
 * suffer the consequences. */
extern ship_t ship;                     /* the spaceship */
extern planet_t planet[NUM_PLANETS];    /* the planets */
extern rel_t rel_pos[NUM_PLANETS];      /* relative position to the ship */
extern rel_t rel_speed[NUM_PLANETS];    /* speed to the ship */
extern cd_t winpos;     /* top left of the window, [world coordinates] */
extern cd_t windim;     /* extent of the window [world coordinates] */

/* All functions that return an integer value return 0 on normal
 * completion. A non-zero return value indicates an error. */

/* Simulation functions */

/* pl_init: This functions initializes the simulation module. It should be
 * called before any other function. */
int pl_init (int screen_width, int screen_height);

/* pl_set_view_screen: Set the screen size. If either width or height are
 * <= 0, the display screen size is not altered. This function must be
 * called first before a view is used. */
int pl_set_view_screen (int width, int height);

/* pl_set_view_center: Set the point in world space where the view is
 * centred. */
void pl_set_view_center (double x, double y);

/* pl_set_view_size: Set the view window size in world coordinates. */
int pl_set_view_size (double width, double height);

/* pl_sim_step: Run a simulation step. */
void pl_sim_step (unsigned seconds);

/* pl_project_to_screen: must be called to update the screen positions. */
void pl_project_to_screen (void);

/* pl_update_rel: update the rel_pos and rel_speed arrays. */
void pl_update_rel (void);

/* Utility functions */

/* pl_set_ship_std_orbit: Put the ship in a standard orbit (= x planetary
 * radius above the surface) around the planet, given by the index in the
 * planet array. */
void pl_set_ship_std_orbit (int t);

/* pl_world_to_screen: Converts coordinates to screen coordinates. */
int pl_world_to_screen (const cd_t *world, ci_t *screen);

/* Calculate the gravity accelleration vector at a given point in
 * space. This works by calculating the gravity accellerations from all
 * planets and adding them. */
cd_t pl_gravity (cd_t pos);

/* pl_distance: Calculate the distance between the positions pointed to by
 * "a" and "b", return the value in the variable pointed to by "res". A
 * normalized vector a->b is returned in "res2".  If any of the parameters
 * "a", "b" or "res" are NULL it returns a non-zero value. Normally it
 * returns 0. */
int pl_distance (const cd_t *a, const cd_t *b, 
                 double *res, cd_t *res2);

/* Calculate relative position or speed, a relative to b (a-b). */
int pl_rel (const cd_t *a, const cd_t *b, rel_t *res);

#endif /* _PLANETS_H */
